<?php

namespace Drupal\views_all_fields\Plugin\views\field;

use Drupal\views\Plugin\views\field\FieldPluginBase;

/**
 * A handler to provide a field that is completely custom by the administrator.
 *
 * @ingroup views_field_handlers
 *
 * @ViewsField("vaf")
 */
class views_handler_field_views_all_fields extends FieldPluginBase {

  function option_definition() {
    $defaults = parent::option_definition();
    $defaults['vaf_fields_exclude'] = array('default' => array());
    $defaults['vaf_bundle'] = array('default' => array());
    return $defaults;
  }

  /**
   * Options form to exclude fields.
   */
  function options_form(&$form, &$form_state) {
    parent::options_form($form, $form_state);

    $form['vaf_bundle'] = array(
      '#title' => t('Bundle'),
      '#description' => t('Pull fields from bundle. Leave blank for all. Resave to see allowed fields.'),
      '#type' => 'select',
      '#multiple' => TRUE,
      '#default_value' => $this->options['vaf_bundle'],
    );

    foreach (entity_get_info() as $type => $info) {
      if ($info['base table'] == $this->table) {
        $entity_type = $type;
        foreach ($info['bundles'] as $key => $bundle_info) {
          $form['vaf_bundle']['#options'][$key] = $bundle_info['label'];
        }
        break;
      }
    }

    $entity_type_fields = field_info_instances($entity_type);
    $fields_in_view = $this->view->get_items('field');

    foreach ($entity_type_fields as $entity_type => $entity_instances) {
      foreach ($entity_instances as $field => $instance) {
        if (!isset($fields_in_view[$field]) && (empty($this->options['vaf_bundle']) || isset($this->options['vaf_bundle'][$instance['bundle']]))) {
          $options[$field] = "{$instance['label']} ({$instance['bundle']}-{$instance['field_name']})";
        }
      }
    }

    asort($options);
    $form['vaf_fields_exclude'] = array(
      '#title' => t('Exclude'),
      '#description' => t('Fields marked here will not be pulled. Note that fields already included in this view are already excluded.'),
      '#type' => 'checkboxes',
      '#options' => $options,
      '#default_value' => $this->options['vaf_fields_exclude'],
    );
  }

}
